@@ -1,4 +1,4 @@
-use v6.c;
+use v6;
 
 =begin pod
 
@@ -32,13 +32,17 @@ my $m = META6.new(   name        => 'META6',
                      provides => {
                         META6 => 'lib/META6.pm',
                      },
-                     license     => 'Artistic',
+                     license     => 'Artistic-2.0',
                      production  => False,
 
                  );
 
 print $m.to-json;
 
+my $m = META6.new(file => './META6.json');
+$m<version description> = v0.0.2, 'Work with Perl 6 META files even better';
+spurt('./META6.json', $m.to-json);
+
 =end code
 
 =head1 DESCRIPTION
@@ -65,7 +69,7 @@ module.
 
 =head2 method new
 
-    multi method new(Str :$file!) 
+    multi method new(Str :$file!)
     multi method new(IO::Path :$file!)
     multi method new(IO::Handle :$file!)
     multi method new(Str:D :$json!)
@@ -92,12 +96,47 @@ representation of the META info. The class should prevent there being
 anything that can't be represented as JSON so it shouldn't throw an
 exception.
 
+=head2 method AT-KEY
+
+    method AT-KEY($key)
+
+Support method to allow subscripts on META6 objects.
+
+=head2 method EXISTS-KEY
+
+    method EXISTS-KEY($key)
+
+Support method to allow subscripts on META6 objects.
+
 =end pod
 
 use JSON::Name;
-use JSON::Class:ver(v0.0.5..*);
+use JSON::Class:ver(v0.0.5+);
 
-class META6:ver<0.0.5>:auth<github:jonathanstowe> does JSON::Class {
+role AutoAssoc {
+    method AT-KEY($key) {
+        self!json-name-to-attibute($key).?get_value(self);
+    }
+
+    method EXISTS-KEY($key --> Bool) {
+        self.AT-KEY($key).defined
+    }
+
+    method ASSIGN-KEY($key, \value) {
+        if self!json-name-to-attibute($key) -> $attr {
+            $attr.set_value(self, value)
+        }
+    }
+
+    method !json-name-to-attibute($json-name) {
+        state %lookup = do for self.^attributes(:local).grep({ $_.has_accessor }) {
+            (.?json-name ?? .json-name !! .name).subst(/^ '$!' | '%!' | '@!' /, '') => $_
+        }
+        %lookup{$json-name}
+    }
+}
+
+class META6:ver<0.0.20>:auth<github:jonathanstowe>:ver<1.0> does JSON::Class does AutoAssoc {
 
     enum Optionality <Mandatory Optional>;
 
@@ -106,7 +145,7 @@ class META6:ver<0.0.5>:auth<github:jonathanstowe> does JSON::Class {
         has Version $.spec-version is rw = Version.new(0);
     }
 
-    role MetaAttribute::Specfication does MetaAttribute {
+    role MetaAttribute::Specification does MetaAttribute {
 
     }
 
@@ -124,7 +163,7 @@ class META6:ver<0.0.5>:auth<github:jonathanstowe> does JSON::Class {
     }
 
     my sub set-specification(Attribute $a, Optionality $optionality = Optional, Version $spec-version = Version.new(0)) {
-        $a does MetaAttribute::Specfication;
+        $a does MetaAttribute::Specification;
         $a.optionality = $optionality // Optional;
         $a.spec-version = $spec-version // Version.new(0);
         $a;
@@ -157,20 +196,27 @@ class META6:ver<0.0.5>:auth<github:jonathanstowe> does JSON::Class {
         self.from-json($json);
     }
 
-    class Support {
-        has Str $.source is rw is specification(Optional);
-        has Str $.bugtracker is rw is specification(Optional);
-        has Str $.email is rw is specification(Optional);
-        has Str $.mailinglist is rw is specification(Optional);
-        has Str $.irc is rw is specification(Optional);
-        has Str $.phone is rw is specification(Optional);
+    multi method new(*%items) {
+        self.bless(|%items);
+    }
+
+    class Support does AutoAssoc {
+        has Str $.source is rw      is specification(Optional);
+        has Str $.bugtracker is rw  is specification(Optional) is json-skip-null;
+        has Str $.email is rw       is specification(Optional) is json-skip-null;
+        has Str $.mailinglist is rw is specification(Optional) is json-skip-null;
+        has Str $.irc is rw         is specification(Optional) is json-skip-null;
+        has Str $.phone is rw       is specification(Optional) is json-skip-null;
+        has Str $.license is rw     is specification(Optional) is json-skip-null;
     }
 
     # cope with "v0.0.1"
-    sub unmarsh-version(Str() $v) returns Version {
+    sub unmarsh-version(Str() $v ) returns Version {
         my $ver = Version.new($v);
         if $ver.parts[0] eq 'v' {
-            $ver.parts.shift;
+            my @parts = $ver.parts;
+            @parts.shift;
+            $ver = Version.new(@parts.join('.'));
             warn 'prefix "v" seen in version string, this may not be what you want' unless $seen-vee;
             $seen-vee = True;
         }
@@ -178,28 +224,31 @@ class META6:ver<0.0.5>:auth<github:jonathanstowe> does JSON::Class {
     }
 
 
-    has Version     $.meta6          is rw is marshalled-by('Str') is unmarshalled-by(&unmarsh-version) is specification(Optional) = Version.new(0);
-    has Version     $.perl          is rw is marshalled-by('Str') is unmarshalled-by(&unmarsh-version) is specification(Mandatory);
+    has Version     $.meta-version  is rw is marshalled-by('Str') is unmarshalled-by(&unmarsh-version) is specification(Optional) = Version.new(0);
+    has Version     $.perl-version  is rw is marshalled-by('Str') is unmarshalled-by(&unmarsh-version) is specification(Mandatory) is json-name('perl');
     has Str         $.name          is rw is specification(Mandatory);
     has Version     $.version       is rw is marshalled-by('Str') is unmarshalled-by(&unmarsh-version) is specification(Mandatory);
     has Str         $.description   is rw is specification(Mandatory);
     has Str         @.authors       is rw is specification(Optional);
-    has Str         $.author        is rw is customary;
+    has Str         $.author        is rw is customary is json-skip-null;
     has Str         %.provides      is rw is specification(Mandatory);
     has Str         @.depends       is rw is specification(Optional);
-    has Str         %.emulates      is rw is specification(Optional);
-    has Str         %.supersedes    is rw is specification(Optional);
-    has Str         %.superseded-by is rw is specification(Optional);
-    has Str         %.excludes      is rw is specification(Optional);
+    has Str         %.emulates      is rw is specification(Optional) is json-skip-null;
+    has Str         %.supersedes    is rw is specification(Optional) is json-skip-null;
+    has Str         %.superseded-by is rw is specification(Optional) is json-skip-null;
+    has Str         %.excludes      is rw is specification(Optional) is json-skip-null;
     has Str         @.build-depends is rw is specification(Optional);
     has Str         @.test-depends  is rw is specification(Optional);
-    has             %.resource      is rw is specification(Optional);
-    has Support     $.support       is rw is specification(Optional);
-    has Bool        $.production    is rw is specification(Optional);
+    has Str         @.resources     is rw is specification(Optional);
+    has Support     $.support       is rw is specification(Optional) = Support.new;
+    has Bool        $.production    is rw is specification(Optional) is json-skip-null;
     has Str         $.license       is rw is specification(Optional);
     has Str         @.tags          is rw is specification(Optional);
     has Str         $.source-url    is rw is customary;
-    has Str         $.auth          is rw is customary;
+    has Str         $.auth          is rw is specification(Optional);
+    has Str         $.api           is rw is specification(Optional) is json-skip-null;
 }
 
+multi sub postcircumfix:<{ }>(META6 \SELF, Iterable \key, Mu \ASSIGN) is raw {}
+
 # vim: expandtab shiftwidth=4 ft=perl6
